use crate::{Image, Tileset, Vec2};
use std::rc::Rc;

//odd-q vertical layout https://www.redblobgames.com/grids/hexagons
pub struct HexMap {
    size: Vec2<i32>,
    data: Vec<i32>,
    tileset: Rc<Tileset>,
}

impl HexMap {
    //pub static
    pub fn new(size: Vec2<i32>, tileset: Rc<Tileset>) -> Self {
        HexMap {
            size,
            data: vec![0; size.size()],
            tileset,
        }
    }

    //pub
    pub fn get(&self, coord: Vec2<i32>) -> i32 {
        self.data[self.coord_to_idx(coord)]
    }

    pub fn set(&mut self, coord: Vec2<i32>, val: i32) {
        let idx = self.coord_to_idx(coord);
        self.data[idx] = val;
    }

    pub fn size(&self) -> Vec2<i32> {
        self.size
    }

    //pubcrate
    pub(crate) fn draw_tile_to_image(
        &self,
        target: &mut Image,
        coord: Vec2<i32>,
        offset: Vec2<i32>,
    ) {
        let x = coord.x * (self.tileset.tile_size().x as f64 * 3.0 / 4.0).ceil() as i32;
        let y =
            coord.y * self.tileset.tile_size().y + (coord.x % 2) * (self.tileset.tile_size().y / 2);
        target.draw_image(
            Vec2 { x, y } + offset,
            self.tileset.get(self.data[self.coord_to_idx(coord)]),
        )
    }

    //priv
    fn coord_to_idx(&self, coord: Vec2<i32>) -> usize {
        (coord.x + coord.y * self.size.x) as usize
    }
}
